/* gnubiff -- a mail notification program
 * Copyright (C) 2000-2003 Nicolas Rougier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#include <sstream>
#include <cstdio>
#include <string>
#include <glib.h>
#include "Applet.h"
#include "Mailbox.h"
#include "Popup.h"



// ===================================================================
// = Static attributes ===============================================
// ===================================================================


// ===================================================================
// = Callbacks "C" binding ===========================================
// ===================================================================
extern "C" {
  //! C binding
  void APPLET_on_button_press (GtkWidget *widget, GdkEventButton *event, gpointer data)	{((Applet *) data)->on_button_press (event);}
  //! C binding
  gboolean APPLET_on_poll (gpointer data) {return ((Applet *) data)->on_poll ();}
}


// ===================================================================
// = Applet =========================================================
// ===================================================================
Applet::Applet (Biff *owner, string xmlFilename) : GUI (xmlFilename) {
  _owner = owner;
  _polltag = 0;
}


// ===================================================================
// = ~Applet ========================================================
// ===================================================================
Applet::~Applet (void) {
}


// ===================================================================
// = start ===========================================================
/**
      Start poll timer
 */
// ===================================================================
void Applet::start (void) {
  stop();
  _polltag = gtk_timeout_add (_owner->_polltime*1000, APPLET_on_poll, this);
}


// ===================================================================
// = stop ============================================================
/**
   Stop poll timer
 */
// ===================================================================
void Applet::stop (void) {
  if (_polltag > 0)
    gtk_timeout_remove (_polltag);
}


// ===================================================================
// = on_poll =========================================================
/**
   Callback on poll timer
   @return true if popup window has not been popped-up (indicating
   that we want the timer to continue) or false if popup window has
   been popped-up (when popup window will be popped-out, timer will be
   restarted)
 */
// ===================================================================
gboolean Applet::on_poll (void) {
  _owner->mailbox()->update();
  _owner->mailbox()->fetch(); 

  if (_owner->mailbox()->status() == MAILBOX_NEW) {
    // Play sound
    if (_owner->_sound_type == SOUND_BEEP)
      gdk_beep ();    
    else if (_owner->_sound_type == SOUND_FILE) {
      stringstream s;
      s << _owner->_sound_volume/100.0;
      string command = "play " + _owner->_sound_file + string (" -v ");
      command += s.str();
      command += " &";
      system (command.c_str());
    }
  }

  update();

  if ((_owner->_display_popup)  && (_owner->mailbox()->status() == MAILBOX_NEW) && (_owner->mailbox()->unreads())) {
    _owner->popup()->update();
    _owner->popup()->show();

    // Here we stop timer since popdown of the popup window will restart it
    return false;
  }

  // Go on with timer
  return true;
}


// ===================================================================
// = dock ============================================================
/**
   Dock function that will specialized later
 */
// ===================================================================
void Applet::dock (GtkWidget *widget) {}
