/* gnubiff -- a mail notification program
 * Copyright (C) 2000-2003 Nicolas Rougier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

// ===================================================================
/**
   @class Applet
   @brief Interface class for a generic applet

   This class represents a generic applet that will be later
   specialized into AppletGTK or AppletGNOME depending on configure
   options. If you want to adapt gnubiff for another window manager
   (and another docking system), this is probably the place you to
   look at.
*/
// ===================================================================

#ifndef _APPLET_H
#define _APPLET_H

#ifdef HAVE_CONFIG_H
#   include "../config/config.h"
#endif
#include "GUI.h"
#include "Biff.h"


// ===================================================================
// - Constant definitions --------------------------------------------
// ===================================================================


using namespace std;
class Applet : public GUI {
  // ===================================================================
  // - Public methods --------------------------------------------------
  // ===================================================================
public:
  //! @name Base functions
  // @{
  Applet (class Biff *owner, string xmlFilename);
  virtual ~Applet (void);
  //@}

  //! @name I/O functions
  // @{
  // @}

  //! @name Object manipulations
  // @{
  virtual void dock (GtkWidget *widget);
  virtual void update (void) = 0;
  void start (void);
  void stop (void);
  gboolean on_poll (void);
  // @}


  //! @name Callbacks
  // @{
  virtual gboolean on_button_press (GdkEventButton *event) = 0;
  // @}


  // ===================================================================
  // - Public attributes -----------------------------------------------
  // ===================================================================
public:



  // ===================================================================
  // - Protected methods -----------------------------------------------
  // ===================================================================
protected:

  // ===================================================================
  // - Protected attributes --------------------------------------------
  // ===================================================================
protected:
  class Biff *		_owner;		/*!< \brief Owner of this interface	*/



  // ===================================================================
  // - Private methods -------------------------------------------------
  // ===================================================================
private:

  // ===================================================================
  // - Private attributes ----------------------------------------------
  // ===================================================================
private:
  gint			_polltag;	/*!< \brief Tag for poll timer */

};

#endif /* _APPLET_H */
