/* gnubiff -- a mail notification program
 * Copyright (C) 2000-2003 Nicolas Rougier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#include <sstream>
#include <cstdio>
#include <string>
#include <glib.h>
#include "AppletGNOME.h"
#include "Mailbox.h"
#include "Popup.h"


// ===================================================================
// = Static attributes ===============================================
// ===================================================================


// ===================================================================
// = Callbacks "C" binding ===========================================
// ===================================================================
extern "C" {
  //! C binding
  gboolean APPLET_GNOME_on_change_orient (GtkWidget *widget, PanelAppletOrient orient, gpointer data)
  {return ((AppletGNOME *) data)->on_change_orient ();}
  //! C binding
  gboolean APPLET_GNOME_on_change_size (GtkWidget *widget, int size, gpointer data)
  {return ((AppletGNOME *) data)->on_change_size ();}
  //! C binding
  gboolean APPLET_GNOME_on_button_press (GtkWidget *widget, GdkEventButton *event, gpointer data)
  {return ((AppletGNOME *) data)->on_button_press (event);}
  //! C binding
  void APPLET_GNOME_on_menu_properties (BonoboUIComponent *uic, gpointer data, const gchar *verbname)
  {((AppletGNOME *) data)->on_menu_properties (uic, verbname);}
  //! C binding
  void APPLET_GNOME_on_menu_mail_app (BonoboUIComponent *uic, gpointer data, const gchar *verbname)
  {((AppletGNOME *) data)->on_menu_mail_app (uic, verbname);}
  //! C binding
  void APPLET_GNOME_on_menu_about (BonoboUIComponent *uic, gpointer data, const gchar *verbname)
  {((AppletGNOME *) data)->on_menu_about (uic, verbname);}
}

// ===================================================================
// = AppletGNOME =====================================================
// ===================================================================
AppletGNOME::AppletGNOME (Biff *owner, string xmlFilename) : Applet (owner, xmlFilename) {
  _about = 0;
}


// ===================================================================
// = ~AppletGNOME ====================================================
// ===================================================================
AppletGNOME::~AppletGNOME (void) {
}


// ===================================================================
// = dock ============================================================
/**
   Dock the applet within gnome panel
 */
// ===================================================================
void AppletGNOME::dock (GtkWidget *applet) {
  static const BonoboUIVerb gnubiffMenuVerbs [] = {
    BONOBO_UI_VERB ("Props",   APPLET_GNOME_on_menu_properties),
    BONOBO_UI_VERB ("MailApp", APPLET_GNOME_on_menu_mail_app),
    BONOBO_UI_VERB ("About",   APPLET_GNOME_on_menu_about), 
    BONOBO_UI_VERB_END
  };
 
  panel_applet_setup_menu_from_file (PANEL_APPLET (applet),
				     NULL,
				     "GNOME_gnubiffApplet.xml",
				     NULL,
				     gnubiffMenuVerbs,
				     this);
  gtk_widget_reparent (get ("table"), applet);  
  g_signal_connect (G_OBJECT (applet),"change_orient",       GTK_SIGNAL_FUNC (APPLET_GNOME_on_change_orient), this);
  g_signal_connect (G_OBJECT (applet),"change_size",         GTK_SIGNAL_FUNC (APPLET_GNOME_on_change_size), this);
  g_signal_connect (G_OBJECT (applet), "button_press_event", GTK_SIGNAL_FUNC (APPLET_GNOME_on_button_press), this);

  gtk_widget_show (applet);

  _gnome_applet = applet;

  return;
}

// ===================================================================
// = update ==========================================================
/**
   Update applet (title, image, labels)
 */
// ===================================================================
void AppletGNOME::update (void) {
  string text;

  text = "<span font_desc=\"";
  text += _owner->_biff_body_font;
  text += "\">";

  if (_owner->mailbox()->status() == MAILBOX_UNITIALIZED)
    text +=  "<span color=\"yellow\">--</span>";
  else if (_owner->mailbox()->status() == MAILBOX_ERROR)
    text +=  "<span color=\"red\">!!</span>";
  else {
    text += "<span color=\"";
    text += _owner->_biff_body_color;
    text += "\"><b>";
    stringstream unreads;
    unreads << _owner->mailbox()->unreads();
    if (_owner->mailbox()->unreads() < _owner->_max_header_display)
      text += unreads.str();
    else
      text += "++";
    text += "</b></span>";
  }
  text += "</span>";

  gtk_label_set_markup (GTK_LABEL(get ("hunread")), text.c_str());
  gtk_label_set_markup (GTK_LABEL(get ("vunread")), text.c_str());

  GdkPixbufAnimation *gdkpix;
  if (_owner->mailbox()->unreads() > 0)
    gdkpix = gtk_image_get_animation (GTK_IMAGE(_owner->setup()->get("mail_image_image")));
  else
    gdkpix = gtk_image_get_animation (GTK_IMAGE(_owner->setup()->get("no_mail_image_image")));

  gtk_image_set_from_animation (GTK_IMAGE (get("himage")), gdkpix);
  gtk_image_set_from_animation (GTK_IMAGE (get("vimage")), gdkpix);

  guint size = panel_applet_get_size (PANEL_APPLET (_gnome_applet));
  PanelAppletOrient orient = panel_applet_get_orient (PANEL_APPLET (_gnome_applet));
  guint width = gdk_pixbuf_animation_get_width (gdkpix);
  guint height = gdk_pixbuf_animation_get_height (gdkpix);
  gboolean too_big = false;

  // The panel is oriented horizontally
  if ((orient == PANEL_APPLET_ORIENT_DOWN) || (orient == PANEL_APPLET_ORIENT_UP)) {
    gtk_widget_hide (get ("vunread"));
    gtk_widget_hide (get ("vimage"));
    gtk_widget_show (get ("hunread"));
    gtk_widget_show (get ("himage"));
    if (height > size) {
      gtk_widget_hide (get ("himage"));
      too_big = true;
    }
  }
  // The panel is oriented vertically
  else {
    gtk_widget_hide (get ("hunread"));
    gtk_widget_hide (get ("himage"));
    gtk_widget_show (get ("vunread"));
    gtk_widget_show (get ("vimage"));
    if (width > size)
      gtk_widget_hide (get ("vimage"));
  }

  if (_owner->_display_title) {
    text = "<span font_desc=\"";
    text += _owner->_biff_title_font;
    text += "\" color=\"";
    text += _owner->_biff_title_color;
    text += "\">";
    text += _owner->_title;
    if (too_big)
      text += ": ";
    text += "</span>";
    gchar *buffer = g_locale_to_utf8 (text.c_str(),-1,0,0,0);
    gtk_label_set_markup (GTK_LABEL(get ("title")), buffer);
    g_free (buffer);
    gtk_widget_show (get ("title"));
  }
  else
    gtk_widget_hide (get ("title"));
}


// ===================================================================
// = show ============================================================
/**
   Show applet.
*/
// ===================================================================
void AppletGNOME::show (string name) {
  gtk_widget_show (_gnome_applet);
}


// ===================================================================
// = hide ============================================================
/**
   Hide applet.
*/
// ===================================================================
void AppletGNOME::hide (string name) {
  gtk_widget_hide (_gnome_applet);
}

// ===================================================================
// = on_change_orient ================================================
/**
   Callback on panel orientation change
   @return TRUE
*/
// ===================================================================
gboolean AppletGNOME::on_change_orient (void) {
  update();
  return TRUE;
}


// ===================================================================
// = on_change_size ==================================================
/**
   Callback on panel size change
   @return TRUE;
*/
// ===================================================================
gboolean AppletGNOME::on_change_size (void) {
  update();
  return TRUE;
}


// ===================================================================
// = on_button_press =================================================
/**
   Callback on button press event
   @param event is the button event
   @return FALSE indicating button press event needs to be handled by
   other widgets
*/
// ===================================================================
gboolean AppletGNOME::on_button_press (GdkEventButton *event) {
  if (event->button == 1) {
    _owner->applet()->stop();
    _owner->mailbox()->update();
    _owner->mailbox()->fetch();
    update();
    if ((_owner->mailbox()->status() == MAILBOX_NEW) || (_owner->mailbox()->status() == MAILBOX_OLD)) {
      _owner->popup()->update();
      _owner->popup()->show();
    }
    else
      _owner->applet()->start();
  }  
  return FALSE;
}


// ===================================================================
// = on_menu_properties ==============================================
/**
   Callback on menu "Preferences"
   @param uic GNOME stuff
   @param verbname GNOME stuff
*/
// ===================================================================
void AppletGNOME::on_menu_properties (BonoboUIComponent *uic, const gchar *verbname) {
  _owner->applet()->stop();
  _owner->popup()->hide();
  _owner->setup()->show();
  if (_about != 0)
    gdk_window_hide (_about->window);
}


// ===================================================================
// = on_menu_mail_app ================================================
/**
   Callback on menu "Start mail client"
   @param uic GNOME stuff
   @param verbname GNOME stuff
*/
// ===================================================================
void AppletGNOME::on_menu_mail_app (BonoboUIComponent *uic, const gchar *verbname) {
  string command = _owner->_mail_app + " &";
  system (command.c_str());
}



// ===================================================================
// = on_menu_mail_app ================================================
/**
   Callback on menu "About"
   @param uic GNOME stuff
   @param verbname GNOME stuff
*/
// ===================================================================
void AppletGNOME::on_menu_about (BonoboUIComponent *uic, const gchar *verbname) {
  static const gchar *authors[] = {
    "Nicolas Rougier <Nicolas.Rougier@loria.fr>",
    NULL
  };
 
  if (_about != 0) {
    gdk_window_show (_about->window);
    gdk_window_raise (_about->window);
    return;
  }
  _about = gnome_about_new ("gnubiff applet", PACKAGE_VERSION,
			   "(C) 2000-2003\n ",
			   _("Released under the GNU general public license.\n"\
			   "Mail notification program."),
			   authors,
			   NULL, NULL, gdk_pixbuf_new_from_file (GNUBIFF_DATADIR"/logo.png", 0));
  g_signal_connect (G_OBJECT (_about), "destroy", G_CALLBACK (gtk_widget_destroyed), &_about);
  gtk_widget_show (_about);
}
