/* gnubiff -- a mail notification program
 * Copyright (C) 2000-2003 Nicolas Rougier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

// ===================================================================
/**
   @class GUI
   @brief Abstract class for generic Graphical User Interface
    
   The GUI class is an abstract class that allow derived classes to
   use basic functions such as creating the interface from a glade
   file, connect callback handlers and manipulating specific widgets
   through the get function.
*/
// ===================================================================

#ifndef _GUI_H
#define _GUI_H

#ifdef HAVE_CONFIG_H
#   include "../config/config.h"
#endif
#include <string>
#include <glib.h>
#include <gtk/gtk.h>
#include <glade/glade.h>



// ===================================================================
// - Constant definitions --------------------------------------------
// ===================================================================

using namespace std;
class GUI {
  // ===================================================================
  // - Public methods --------------------------------------------------
  // ===================================================================
public:
  //! @name Base methods
  //@{
  GUI (string xmlFilename);
  virtual ~GUI (void);
  //@}

  //! @name Object manipulations
  // @{
  virtual gint create (void);
  virtual void show (string name = "dialog");
  virtual void hide (string name = "dialog");
  string utf8_to_filename (string text);
  string utf8_to_locale (string text);
  string filename_to_utf8 (string text);
  string locale_to_utf8 (string text);
  //@}

  //! @name Attributes consultations / modifications
  // @{
  GtkWidget *	get (string name);
  //@}

  //! @name Callbacks
  // @{
  virtual gboolean on_delete (GtkWidget *widget,  GdkEvent *event);
  virtual gboolean on_destroy (GtkWidget *widget, GdkEvent *event);
  //@}


  // ===================================================================
  // - Public attributes -----------------------------------------------
  // ===================================================================
public:



  // ===================================================================
  // - Protected methods -----------------------------------------------
  // ===================================================================
protected:


  // ===================================================================
  // - Protected attributes --------------------------------------------
  // ===================================================================
protected:
  GladeXML *		_xml;		/*!< \brief Interface description using XML/glade-2 file */
  string		_xmlFilename;	/*!< \brief Name of the interface file (glade-2 format) */



  // ===================================================================
  // - Private methods -------------------------------------------------
  // ===================================================================
private:


  // ===================================================================
  // - Private attributes ----------------------------------------------
  // ===================================================================
private:
};

#endif /* _GUI_H */
